#!/stf/sys/bin/perl

# converts alignment into IMV-accepted formats
# can be used as a run-thru service taking a parameter 'aln_filename' for the MSF file path on server

use strict;
use CGI qw/:standard -debug/;
use IO::String;
use Bio::AlignIO;
use Data::Dumper;
use JSON;

###############################################################################
# Configurable section, must set the paths!
my %tmp = (
  hd => '/absolute/file/system/path/alignments', # the path to put the converted json file
  web => '/URL/path/alignments', # the web path for the converted json file
  viewer => 'http://full/URL/path/PubD-IMV/AlignViewer/map.html' # the url for alignment viewer for the json file
);
my $inputfile = param('aln_filename'); # useful for run-thru as mentioned above
my $user = $ENV{LDAP_USER} || 'myuserid'; # this is optional
###############################################################################

# to test/debug this script, put in the file path below and run the script directly on command line:
# process('/tmp/aln2imv_3381320'); # this does the conversion using the msf file path
# exit;

print "Content-type: text/html\n\n" unless $inputfile;

if(param('aln_sub'))
{
  my $filename = param('uploaded_file');
  my $f = upload('aln_file');
  open($f, $filename) unless $f;
  if (defined $f)
  {
    my $buffer;
    my $tmpfilename = '/tmp/aln2imv_'.$user.'_'.time;
    open(OUTFILE, ">$tmpfilename");
    while(read($f,$buffer,1024))
    {
      print OUTFILE $buffer;
    }
    close $f;
    close(OUTFILE);
    process($tmpfilename, param('ref'));
  }
}
elsif($inputfile) # used by Charles' AlignThis
{
  process($inputfile, param('ref'));
}
print start_form,
      "Please select an alignment file in MSF format:<br/>\n",
      filefield('aln_file','C:\\test.msf',50,80),
      "<br>Enter the id for the sequence that should serve as consensus (optional):",
      textfield('ref'),br,
      submit('aln_sub','Submit'),
      end_form;

sub process
{
  my ($file, $ref) = @_;
  my $in = Bio::AlignIO->new(-file => $file, -format => 'msf');
  while (my $aln = $in->next_aln())
  {
    my $cons = $aln->consensus_string(50);
    my (@tracks, $track, $base, $alpha);
    foreach my $seq ($aln->each_seq)
    {
      $alpha = (($seq->alphabet eq 'protein')? 'prot' : 'dna') unless $alpha;
      my $isBase = $ref eq $seq->id;
      my $s = $seq->seq;
      $s =~ tr/\./-/;
      $track = {
        tclass => $alpha,
        id => $seq->id,
        type => $isBase? 'base' : 'mutation',
        seq => [{ s => $s }]
      };
      if($isBase) { $base = $track; }
      else { push(@tracks, $track); }
    }
    $base = {
      tclass => $alpha,
      id => 'concensus',
      type => 'base',
      seq => [{ s => $cons }]
    } unless $base;
    unshift(@tracks, $base);
    # now process all non-base seqs to produce mutation info
    my @baseseq = split(//, $base->{seq}[0]{s});
    for(my $i = 1; $i < @tracks; $i++)
    {
      my @seq = split(//, $tracks[$i]{seq}[0]{s});
      for(my $j = 0; $j < @baseseq; $j++)
      {
        if($seq[$j] ne $baseseq[$j])
        {
          push(@{$tracks[$i]{seq}}, {
            tx => $j + 1,
            b => [[$seq[$j],100]]
          });
        }
      }
      shift @{$tracks[$i]{seq}};
    }
    my $f = "$user\_".time.".json";
    open(OUT, ">$tmp{hd}/$f") || die "can't open $tmp{hd}/$f - $!\n";
    print OUT to_json({seqdata=>\@tracks}, {utf8 => 1, pretty => 1});
    close(OUT);
    my $url = "$tmp{viewer}?path=$tmp{web}/$f&tracksOn=$alpha";
    if($inputfile)
    {
      print $url;
      exit;
    }
    print a({href=>$url},"Click to view the alignment"), br, br, br;
  }
}