/////////////////////////////////////////////////////////////////////////////////////////////////
// object for the filters for this application, the callback function to be used by dynamicViewer
filterObj = {
  fieldTypes :
    {
      'Mixed Locations':'range',
      'Top mutation %':'range',
      'Min mutation %':'range',
      'Min group SD':'range',
      'ClinicalSubject':'text',
      'Requisition':'text',
      'User':'text',
      'Comment':'text',
      'SampleDescription':'text',
      'ClinicalTimePoint':'text'
    },
  fieldIdNames:
    {
      'mixloc':'Mixed Locations',
      'subject':'ClinicalSubject',
      'req':'Requisition',
      'user':'User',
      'com':'Comment',
      'clindesc':'SampleDescription',
      'timepnt':'ClinicalTimePoint',
      'mut':'Top mutation %',
      'mmut':'Min mutation %',
      'pmut':'Min group SD'
    },
  helpStrings:
    {
      'mixloc':'The number of mixed locations (positions where there are at least 2 possible alleles) automatically predicted in a sequence. Setting a value of ">=10", for example, will only show samples that have at least 10 mixed locations predicted',
      'req':'The CSF Requisition ID, for example RQ-20100105-00005. Entering a value will show only those samples where the requisition matches your request',
      'subject':'Patient ID',
      'user':'Your Windows Username',
      'timepnt':'Clinical Time Point',
      'mut':'Show only locations whose top mutant (mutant with highest %) matches the value/range you enter here. If you enter ">=20", the filter will hide any location that does not have any variant with >=20% mutational rate; if you enter "<5", then only locations whose mutations are all less than 5% would show',
      'mmut':'NOTE: This option is used by "Export Sequences", "Export Mutations", "Export Alignment" options, and "FastaSeq"/"MutantSeq" tabs. When using these options/tabs, do NOT use a range like ">3" here. Instead, only use numbers like "3"!!<br><br>When value here is used for filtering purposes (not the options/tabs above), then for every mixed location, filters out variants whose mutation % is smaller than the value you enter; if there is 0 variant meeting the criteria, hide the location from view; otherwise display either "X" or the actual qualifying variants as the space permits. The reference base/a.a., if qualified, will be in blue color',
      'pmut':'For every location of each sequence group (i.e., samples from the same patient), calculate the S.D.s for each base/a.a. for BL (baseline) mutations and pick the max SD; get the max SD for non-BL mutations in the same way except that the BL average (not non-BL average) is used to accentuate difference between non-BL and BL samples. Then compare the 2 max SDs with the value you enter. If any SD is bigger than your value (meaning enough mutation % change inside the patient\' samples), the location is shown; otherwise it is hidden'
    },
  fieldIds: // used for adding tips
    {
      'CSF':new Array('req','mixloc'),
      'Clinical':new Array('subject','clindesc','timepnt'),
      'Other':new Array('user','com'),
      'Mutation Filters':new Array('mut','mmut','pmut')
    },
  tipsAdded: {},
  tipStrings: { text:'Any character', range: 'Range/number like "3", ">= 0.3", "< 3.5 & >2.8", "1>| 5|=7|>9"', number: 'Numbers only, like "0.3", "3" etc.' },
  featureFields:
    {
      'Top mutation %':'<', // meaning if user only typed a number into filter text box, attach this symbol automatically
      'Min mutation %':'>',
      'Min group SD':'>'
    },
  validateNum : function(val)
  {
    if(val.match(/^\s*$/)) return true;
    if(val.match(/^\s*(>|<|>=|<=|=)?\s*-?[0-9]+(\.[0-9]+)?(\s*&\s*(>|<|>=|<=|=)?\s*-?[0-9]+(\.[0-9]+)?)?\s*$/))
      return true;
    if(val.match(/^\s*-?[0-9]+(\.[0-9]+)?(\|-?[0-9]+(\.[0-9]+)?)*\s*$/)) // allow choosing multiple values
      return true;
    return false;
  },
  createFormTips : function(e) // lazy rendering (demanded also by Ext's lazy rendering)
  {
    var t = e.title;
    if(!this.tipsAdded[t])
    {
      for(var i = 0; i < this.fieldIds[t].length; i++)
      {
        new Ext.ToolTip({
            target: this.fieldIds[t][i],
            dismissDelay: 0,
            mouseOffset: [15, this.fieldIds[t][i]=='pmut'?-140:-55], // avoid conflict with validator message
            html: 'Allowed input:' + this.tipStrings[this.fieldTypes[this.fieldIdNames[this.fieldIds[t][i]]]] +
                  (this.helpStrings[this.fieldIds[t][i]]? '<br><br>' + this.helpStrings[this.fieldIds[t][i]] : '')
        });
        if(displayHint) // not ideal
        {
          var t1 = Ext.getCmp(this.fieldIds[t][i]).el;
          t1.on('dblclick', displayHint);
        }
      }
      this.tipsAdded[t] = true;
    }
  },
  submitFilters : function(e)
  {
    if(!dynamicViewer.data) return;
    this.form = Ext.ComponentMgr.get('filters').getForm();
    if(!this.button) this.button = Ext.ComponentMgr.get('reset');
    var fs = new Array();
    var formValues = this.form.getValues(false), hasFeature = false, hasComment = false, hasGroupFeature = false;
    this.formValues = {};
    this.featureValues = {};
    for(var a in formValues)
      if(formValues[a] && formValues[a].match(/\S/))
      {
        fs.push(a + ':' + formValues[a]);
        if(this.featureFields[a])
        {
          this.featureValues[a] = ((!formValues[a].match(/[<>=]/))? this.featureFields[a] : '') + formValues[a];
          hasFeature = true;
          if(a == 'Min group SD') hasGroupFeature = true;
        }
        else
        {
          this.formValues[a] = formValues[a];
          hasComment = true;
        }
      }
    this.tooltip = this.button.setTooltip('Reset these filters:<br>&nbsp;&nbsp;&nbsp;' + fs.join('<br>&nbsp;&nbsp;&nbsp;'));
    if(this.form.isValid() && this.form.isDirty())
    {
      if(logger) logger.add('Filter', 'Criteria: ' + fs.join(';'));
      if(hasComment)
        dynamicViewer.filterComment({fn:this.commentCallback, scope:this}, true); // true means do not filter feature in dynamicViewer._drawTracks()
      if(hasFeature)
      {
        if(!hasComment && dynamicViewer.featureFilter) // data view needs to be reset to get rid of previous feature filter view
        {
          delete dynamicViewer.featureFilter;
          dynamicViewer.refreshView();
        }
        if(hasGroupFeature) dynamicViewer.trackManager.groupPositionSDs();
        dynamicViewer.filterFeature({fn:this.featureCallback, scope:this});
      }
    }
    else
      BMS.AG.ML.Util.alert('Warning', 'You need to enter some criteria first!');
  },
  // returns true if filter condition's met on the comment that's passed in
  // comment is in the form of: comment[tagName][context] = value array
  commentCallback : function(comment)
  {
    for(var a in this.formValues)
    {
      if(comment[a])
      {
        var com = comment[a], m = false;
        for(var c in com)
        {
          for(var j = 0; j < com[c].length; j++)
          {
            var v = com[c][j];
            if(this.fieldTypes[a] == 'number')
            {
              var parts = this.formValues[a].split(/\|/), op;
              if(parts.length > 1) op = '||';
              else
              {
                parts = this.formValues[a].split(/\&/);
                if(parts.length > 1) op = '&&';
              }
              for(var i = 0; i < parts.length; i++)
              {
                if(parts[i].match(/[<>=]\s*\d/)) parts[i] = v + parts[i];
                else if(parts[i].match(/\d\s*[<>=]/)) parts[i] = parts[i] + v;
                else parts[i] = v + '==' + parts[i];
              }
              if(eval(parts.join(op))) m = true;
            }
            else if((v+'').match(this.formValues[a]))
              m = true;
          }
        }
        if(!m) return false;
      }
      else return false;
    }
    return true;
  },
  // returns true if filter condition's met on the feature that's passed in
  // feature is in the form of: comment[tagName][context] = value array
  featureCallback : function(feature, t) // t = track
  {
    var showFeat = new Array(), found = false;
    var hasShowMut = false, hasTopMut = false, hasAllMut = false, hasPerMut = false;
    for(var a in this.featureValues)
    {
      if(a == 'Top mutation %')
      {
        hasTopMut = true;
        if(!found && feature.b.length && feature.b[0][0] != '.') // special char
        {
          var v = 0;
          for(var i = 0; i < feature.b.length; i++)
            if(v < feature.b[i][1]) v = feature.b[i][1];
          var tmp = eval(v + this.featureValues[a].replace('&', ' && ' + v));
          if(tmp) found = true;
        }
      }
      else if(a == 'Min mutation %')
      {
        hasShowMut = true;
        var per = 100;
        for(var i = 0; i < feature.b.length; i++)
        {
          if(feature.b[0][0] == '.') continue; // special char
          var v = feature.b[i][1];
          per -= v;
          if(eval(v + this.featureValues[a].replace('&', ' && ' + v)))
            showFeat.push(feature.b[i]);
        }
        if(showFeat.length)
        {
          // users want original displayed if it passes test
          if(eval(per + this.featureValues[a].replace('&', ' && ' + per)))
            showFeat.push(['<font style="color:blue;">'+feature.orig[0]+'</font>', per]);
        }
      }
      else if(a == 'Min group SD')
      {
        hasPerMut = true;
        var key = t.group + '-' + t.tclass + '-' + (feature.x || feature.tx);
        var sd = dynamicViewer.trackManager.groupSDs[key];
        var v = 0;
        if(sd)
        {
          if(sd.nb && sd.nb.sd > v) v = sd.nb.sd;
//           if(sd.nn && sd.nn.sd > v) v = sd.nn.sd;
          if(sd.nt && sd.nt.sd > v) v = sd.nt.sd;
        }
        var tmp = eval(v + this.featureValues[a].replace('&', ' && ' + v));
        if(!tmp) return false;
      }
    }
    if(hasTopMut && !found) return false;
    if(hasShowMut)
    {
      var arr = new Array();
      showFeat.sort(function(a,b){return b[1]-a[1]});
      for(var i = 0; i < showFeat.length; i++)
        arr.push(showFeat[i][0]);
      return arr;
    }
    return true; // hasTopMut && found
  }
}