// This class assumes that main app already set appropriate provider (preferrably Ext.ux.state.BMSProvider) for Ext.state.Manager!!
// config format:
// {
//   key:'a unique key for the application that uses dynamicViewer' //required
// }
BMS.AG.ML.DynamicViewer.stateManager = function(config, viewer)
{
  this.parent = viewer;
  this.version = ''; // views on the same data should be shared cross-version, so no need for it now
  this.appConfigID = 'viewerApp' + this.version; // appConfig is config shared among entire app (config is specific for each map)
  if(config.key) this.setKey(config.key);
  this.clearOldCookies(); // legacy
  this.provider = Ext.state.Manager.getProvider();
};
BMS.AG.ML.DynamicViewer.stateManager.prototype =
{
  clearOldCookies: function()
  {
    var a = document.cookie.split(";");
    for (var i=0; i < a.length; i++)
    {
      var p = a[i].split("=");
      if(p[0].match(/^\s*(ys-.+0\.2)$/))
        Ext.util.Cookies.set(RegExp.$1, null, new Date("January 1, 1970")); // another long-standing Ext bug requires us to call set instead of clear
    }
  },
  setKey: function(key)
  {
    this.configID = key + this.version;
  },
  recordState: function()
  {
    this.encodeValue();
    if(!this.travelInHistory)
    {
      var h = new Array(), c = new Array();
      if(this.configHistory && this.configHistory.length)
        for(var i = 0; i <= this.configIdx; i++)
        {
          h.push(this.configHistory[i]);
          c.push(this.controllerRefresh[i]);
        }
      h.push(BMS.AG.ML.Util.clone(this.config));
      c.push(this.config.newControllerData);
      this.configIdx = h.length - 1;
      this.configHistory = h;
      this.controllerRefresh = c;
    }
    this.travelInHistory = false;
  },
  decodeValue: function()
  {
    return {conf:this.provider.get(this.configID)||{}, app:this.provider.get(this.appConfigID)||{}};
  },
  encodeValue: function()
  {
    if(this.config) this.provider.set(this.configID, this.config);
    if(this.appConfig) this.provider.set(this.appConfigID, this.appConfig);
  },
  loadState: function()
  {
    // once cookie exists, one can call either Ext.state.Manager.get or cookie.get
    var a = this.decodeValue();
    var config = a.conf, appConfig = a.app;
    var defConf =
    {
      databox:
      {
        xstart:1,
        ystart:1
      },
      size:8, // only used when user uses this app for 1st time or just after resetting settings
      displayTracks:
      {
        dna:{show:1},
        prot:{show:1},
        dnacoords:{show:1},
        protcoords:{show:1}
      },
      dataurl:this.parent.dataurl,
      dataparam:this.parent.dataparam
    };
    var defAppConf =
    {
      clickToPopup: 1, // whether to popup comment tooltip after user clicked (if 0, mouseover -> popup)
      showSep: 1, // show separators between non-base tracks with different "group" properties
      useCross: 0 // whether to display a cross at 1px distance from where mouse is (this would allow easy alignment of current mouse position with n.t. and row)
    };
    if(!this.config) this.config = {};
    if(!this.appConfig) this.appConfig = {};
    Ext.apply(this.config, config, defConf);
    Ext.apply(this.appConfig, appConfig, defAppConf);
  },
  // back one step in history
  backHistory: function()
  {
    if(this.configIdx > 0 && this.configHistory)
    {
      this.config = this.configHistory[--this.configIdx];
      this.travelInHistory = true;
      this.config.newControllerData = this.controllerRefresh[this.configIdx + 1];
      this.parent.getDataDisplay();
      this.parent.log('Action', 'Move back in viewer history');
    }
    else
      // the following should be handled in app might be
      BMS.AG.ML.Util.alert("The Start of History",'The current view is the start of your view history');
  },
  // forward one step in history
  forwardHistory: function()
  {
    if(this.configHistory && this.configIdx < this.configHistory.length - 1)
    {
      this.config = this.configHistory[++this.configIdx];
      this.travelInHistory = true;
      this.config.newControllerData = this.controllerRefresh[this.configIdx];
      this.parent.getDataDisplay();
      this.parent.log('Action', 'Move forward in viewer history');
    }
    else
      BMS.AG.ML.Util.alert("The End of History",'The current view is the latest in your view history');
  },
  clearState: function()
  {
    this.provider.clear(this.configID);
  },
  resetState: function(refreshController)
  {
    if(refreshController) this.parent.newControllerData = true;
    this.provider.clear(this.configID);
    this.loadState();
    this.parent.getDataDisplay();
  }
};