BMS.AG.ML.DynamicViewer.TrackManager = function(tracks, parent)
{
  this.tracks = {
    base: new Array(),
    nonbase: new Array(),
    all: new Array()
  };
  if(tracks) this.addTracks(tracks);
  this.parent = parent;
};

////////////////////////////////////////////////////////////
// trackmanager stores tracks and manages coordinates space
BMS.AG.ML.DynamicViewer.TrackManager.prototype =
{
  // export all sequences of type ('align', 'fasta', or 'mutant') of tclass ('dna' or 'prot')
  exportSeqs: function(type, tclass)
  {
    var maxWidth = { dna:0, prot:0 }, seqs = { dna:new Array(), prot:new Array() }, names = { dna:new Array(), prot:new Array() };
    var rate = MutationViewer.getMinRate();
    if(this.tracks && this.tracks.all)
    {
      var ts = this.tracks.all;
      for(var i = 0; i < ts.length; i++)
      {
        if(!seqs[ts[i].tclass] || ts[i].noDisplay()) continue;
        seqs[ts[i].tclass].push(type == 'align'? ts[i].getAlignSeq(rate) : type == 'fasta'? ts[i].getSeq(rate) : type == 'mutant'? ts[i].assembleMutSeq(rate): '');
        names[ts[i].tclass].push(ts[i].name);
        if(maxWidth[ts[i].tclass] < ts[i].name.length) maxWidth[ts[i].tclass] = ts[i].name.length;
      }
    }
    var result = { dna:new Array(), prot:new Array() };
    for(var i = 0; i < seqs.dna.length; i++)
      result.dna.push(type=='align'? names.dna[i].pad(maxWidth.dna + 2):'>'+names.dna[i]+'\n', seqs.dna[i], '\n');
    for(var i = 0; i < seqs.prot.length; i++)
      result.prot.push(type=='align'? names.prot[i].pad(maxWidth.prot + 2):'>'+names.prot[i]+'\n', seqs.prot[i], '\n');
    var tmp = 'Minimum Mutation % used to generate the sequences: '+ (rate === undefined? 20:rate) + '\n';
    if(tclass) return tmp + result[tclass].join('');
    else return tmp + result.dna.join('') + '\n\n\n' + result.prot.join('');
  },
  addTracks: function(tracks)
  {
    if(typeof(tracks)== 'object')
    {
      if(!tracks.length) tracks = new Array(tracks);
      this._process(tracks);
    }
    else
    {
      BMS.AG.ML.Util.alert('Error!!', 'No data found for the viewer!');
      return;
    }
    var ts = this.tracks.all;
    for(var i = 0; i < ts.length; i++)
    {
      ts[i].assembleSeq();
      ts[i].setGaps();
    }
  },
  countDisplayedTracks: function(type) // 0: all, 1: base, 2: non-base
  {
    var tracks = 0;
    var ts = type == 2? this.tracks.nonbase : type == 1? this.tracks.base : this.tracks.all;
    for(var i = 0; i < ts.length; i++)
      if(!ts[i].noDisplay()) tracks++;
    return tracks;
  },
  getTrack: function(id)
  {
    for(var i = 0; i < this.tracks.all.length; i++)
      if(this.tracks.all[i].id == id) return this.tracks.all[i];
  },
  getBaseForClass: function(tclass)
  {
    for(var i = 0; i < this.tracks.base.length; i++)
      if(this.tracks.base[i].tclass == tclass) return this.tracks.base[i];
  },
  // create tracks, set base tracks, set original n.t./a.a. for mutations
  _process: function(tracks)
  {
    var base = {};
    for(var i = 0; i < tracks.length; i++)
    {
      // if user forgot to add config info, add it
      if(!this.parent.stateManager.config.displayTracks[tracks[i].tclass])
        this.parent.stateManager.config.displayTracks[tracks[i].tclass] = {show:true};
      var t = new BMS.AG.ML.DynamicViewer.Track(tracks[i], this);
      if(tracks[i].type == 'base')
      {
        this.tracks.base.push(t);
        t.base = t;
        base[t.tclass] = t;
      }
      else
      {
        t.base = base[t.tclass];
        this.tracks.nonbase.push(t);
        t.setOrig();
      }
      this.tracks.all.push(t);
    }
  },
  filterBases: function(type, position, bases)
  {
    var tracks = new Array();
    for(var i = 0; i < this.tracks.nonbase.length; i++)
    {
      var t = this.tracks.nonbase[i];
      if(!t.failedFilter && t.tclass == type)
      {
        var found = false, sum = 0;
        for(var j = 0; j < t.seq.length; j++)
        {
          var x = t.seq[j].x || t.seq[j].tx;
          if(x == position)
          {
            for(var k = 0; k < t.seq[j].b.length; k++)
            {
              if(bases[t.seq[j].b[k][0]]) found = true;
              sum += t.seq[j].b[k][1];
            }
          }
          if(found) break;
        }
        if(!found && sum < 100) // meaning original base is at the position
          found = bases[t.getBaseSeqAt(position, true)];
        if(found) tracks.push(t);
        else
        {
          t.failedFilter = true;
          if(i && type == 'prot' && this.tracks.nonbase[i-1].tclass == 'dna') this.tracks.nonbase[i-1].failedFilter = true; // we count on that protein track immediately follows its DNA track
          if(type == 'dna' && i < this.tracks.nonbase.length && this.tracks.nonbase[i+1].tclass == 'prot') this.tracks.nonbase[i+1].failedFilter = true; // we count on that protein track immediately follows its DNA track
        }
      }
    }
    return tracks;
  },
  // for each position per group per tclass, get the S.D. for BL, nonBL, and nonBL relative to BL average
  // the hash returned is keyed by 'group-tclass-x' (gapped x)
  groupPositionSDs: function(refresh)
  {
    if(this.groupSDs && !refresh) return;
    var conf = this.parent.stateManager.config, types = {dna:1,prot:1}, stats = {}, info = {};
    for(var m in types)
    {
      var origs = {};
      // get all mutations
      for(var i = 0; i < this.tracks.nonbase.length; i++)
      {
        var t = this.tracks.nonbase[i];
        var bl = t.isBL()? 'BL':'NBL';
        var key = t.group+'-'+m;
        if(t.tclass == m)
        {
          var n = {};
          for(var j = 0; j < t.seq.length; j++)
          {
            var x = t.seq[j].x || t.seq[j].tx;
            if(!origs[x]) origs[x] = t.seq[j].orig[0];
            if(t.seq[j].b && t.seq[j].b.length)
            {
              if(!info[key]) info[key] = {};
              if(!info[key][x]) info[key][x] = {ids:{}};
              info[key][x].ids[t.id] = 1;
              if(!info[key][x][bl]) info[key][x][bl] = [];
              var n = t.seq[j].b, s = {};
              // special char
              if(n[0][0] != ' ') // skip 'no data' tracks
              {
                var sum = 0;
                for(var k = 0; k < n.length; k++)
                  if(n[k][0] != '.')
                  {
                    s[n[k][0]] = n[k][1];
                    sum += n[k][1];
                  }
                if(sum < 100) s[origs[x]] = 100 - sum;
                info[key][x][bl].push(s);
              }
            }
          }
        }
      }
      // for all group-tclass-x, we need to add non-mutations too (reference base/a.a.)
      for(var i = 0; i < this.tracks.nonbase.length; i++)
      {
        var t = this.tracks.nonbase[i];
        if(t.tclass == m)
        {
          var key = t.group+'-'+m;
          if(info[key])
          {
            var bl = t.isBL()? 'BL':'NBL';
            for(var x in info[key])
              if(!info[key][x].ids[t.id])
              {
                var s = {};
                s[origs[x]] = 100;
                if(!info[key][x][bl]) info[key][x][bl] = [];
                info[key][x][bl].push(s);
              }
          }
        }
      }
      // info collected for all mutations positions on ALL (including non-mutated) tracks, process position
      for(var key in info)
      {
        for(var x in info[key])
        {
          var sd = {};
          if(info[key][x].BL) sd.nb = this._maxSD(info[key][x].BL);
          if(info[key][x].NBL) sd.nn = this._maxSD(info[key][x].NBL);
          if(info[key][x].BL && info[key][x].NBL) sd.nt = this._maxSD(info[key][x].NBL, sd.nb.avg);
          stats[key+'-'+x] = sd;
        }
      }
    }
    this.groupSDs = stats;
    return stats;
  },
  _maxSD: function(d, blAvg)
  {
    var keys = {}, a = [];
    for(var i = 0; i < d.length; i++)
      for(var j in d[i])
        keys[j] = 1;
    for(var j in keys) a.push(j);
    var maxSD = 0, avg = {};
    for(var i = 0; i < a.length; i++)
    {
      var t = [];
      for(var j = 0; j < d.length; j++)
        t.push(d[j][a[i][0]] || 0);
      var k = BMS.AG.ML.Util.arrayStats(t, {}, blAvg? blAvg[a[i][0]] || 0 : null);
      if(maxSD < k.sd) maxSD = k.sd;
      avg[a[i][0]] = k.avg;
    }
    return {sd:maxSD, avg:avg};
  },
  // position should be gapped, starts at 1!! (array of gapped positions work too)
  // when specifying position, type should always be specified!!
  positionStats: function(type, position, groupid)
  {
    if(!position) return;
    var p = typeof(position) == 'object'? position : [position], pos = {};
    p.sort(function(a,b){return a-b});
    for(var i = 0; i < p.length; i++)
      pos[p[i]] = 1;
    var conf = this.parent.stateManager.config, types = {dna:1,prot:1}, stats = {};
    for(var m in types)
    {
      if((type && type != m) || !conf.displayTracks[m].show) continue;
      var base, tracks = new Array();
      // get all the base nt/aa
      for(var i = 0; i < this.tracks.base.length; i++)
      {
        var t = this.tracks.base[i];
        if(t.tclass == m) { base = t.seq[0].s.split(''); break; } // base is 0-based
      }
      // get all the track seqs as a 2D array
      for(var i = 0; i < this.tracks.nonbase.length; i++)
      {
        var t = this.tracks.nonbase[i];
        if(!t.failedFilter && t.tclass == m && (!groupid || groupid == t.group))
        {
          var n = {};
          for(var j = 0; j < t.seq.length; j++)
          {
            var x = t.seq[j].x || t.seq[j].tx;
            if(pos[x]) n[x] = t.seq[j].b; // n[x] is 1-based
          }
          tracks.push(n);
        }
      }
      // process each position
      stats[m] = new Array();
      var allStats = { base:'', all:{} };
      for(var j = 0; j < tracks.length; j++) // go through each track
      {
        var stat, allPos = [];
        for(var l = 0; l < p.length; l++) // go through each position, get each base/%
        {
          var i = p[l];
          stat = { base:base[i-1], all:{} };
          var n = tracks[j][i], sum = 0;
          if(n && n.length)
          {
            for(var k = 0; k < n.length; k++)
            {
              if(n[k][0] == '.') continue; // special char
              stat.all[n[k][0]] = n[k][1];
              sum += n[k][1];
            }
          }
          if(sum < 100) stat.all[base[i-1]] = 100 - sum;
          allPos.push(stat);
        }
        if(p.length > 1) // combine the positions together
        {
          stat = { base:'', all:{} };
          for(var l = 0; l < p.length; l++)
          {
            var s = allPos[l];
            stat.base += s.base;
            if(!l) stat.all = s.all;
            else
            {
              var current = {};
              for(var k in s.all) // a base at current position
              {
                for(var i in stat.all) // (multi)base accumulated from previously-processed positions
                  current[i+k] = stat.all[i] * s.all[k]/100;
              }
              stat.all = current;
            }
          }
        }
        // combine the stats info for each track
        if(!allStats.base) allStats.base = stat.base;
        for(var i in stat.all)
        {
          if(!allStats.all[i])
            allStats.all[i] = {count:1, percent:[stat.all[i]]};
          else
          {
            allStats.all[i].count++;
            allStats.all[i].percent.push(stat.all[i]);
          }
        }
      }
      // now calculate stats
      for(var i in allStats.all)
      {
        var k = allStats.all[i];
        BMS.AG.ML.Util.arrayStats(k.percent, k);
        k.avg = Math.round(k.avg);
        k.sd = Math.round(k.sd);
      }
      stats[m] = allStats;
    }
    this.stats = stats;
    return stats;
  },
  getDisplayedTracks: function()
  {
    var tracks = new Array(), numtracks = 0, newtrack = 0, conf = this.parent.stateManager.config;
    for(var i = 0; i < this.tracks.base.length; i++)
    {
      var t = this.tracks.base[i];
      if(!conf.displayTracks[t.tclass].show) continue;
      tracks.push(t);
      numtracks++;
    }
    for(var i = 0; i < this.tracks.nonbase.length; i++)
    {
      var t = this.tracks.nonbase[i];
      if(t.noDisplay() || ++newtrack < conf.databox.ystart) continue;
      tracks.push(t);
      if(++numtracks > conf.databox.height) break;
    }
    this.displayedTracks = tracks;
    this.widthLCF = this.getWidthLcf();
    return tracks;
  },
  getSepPositions: function()
  {
    var arr = new Array(), t = this.getDisplayedTracks(), lastgroup = 0;
    for(var i = 0; i < t.length; i++)
    {
      if(t[i].type != 'base' && t[i-1].type != 'base' && t[i].group != lastgroup)
        arr.push(i);
      lastgroup = t[i].group;
    }
    return arr;
  },
  get1stDisplayedTrack: function(norefresh)
  {
    var tracks = norefresh? this.displayedTracks : this.getDisplayedTracks();
    for(var i = 0; i < tracks.length; i++)
    {
      var t = tracks[i];
      if(t.type != 'base' && !t.noDisplay()) return t;
    }
  },
  // get the number (ystart) of the first track that should be displayed
  get1stDisplayedTrackNumber: function()
  {
    return this.getTrackNumberByName(this.get1stDisplayedTrack(true).name);
  },
  // get the number (ystart) for the track with given name
  getTrackNumberByName: function(name, track)
  {
    var newtrack = 0;
    for(var i = 0; i < this.tracks.nonbase.length; i++)
    {
      var t = this.tracks.nonbase[i];
      if(t.noDisplay()) continue;
      ++newtrack;
      if(t.name == name && (!track || t.tclass == track)) return newtrack;
    }
  },
  // since we're only dealing with DNA and Protein (domain is protein) no need to calculate lcf
  getWidthLcf: function(exclude)
  {
    var minwidth = 0, conf = this.parent.stateManager.config;
    for(var i = 0; i < this.tracks.base.length; i++)
    {
      var t = this.tracks.base[i];
      if(t.noDisplay() || t.tclass == exclude) continue;
      var w = t.width || t._guessWidth(t.tclass);
      if(w == 1) return 1;
      if(minwidth == 0 || minwidth > w) minwidth = w;
    }
    return minwidth;
  }
};
